-- Create free recipe for each fluid.
for _, fluid in pairs(data.raw["fluid"]) do
	data:extend(
	{
		{
			type = "recipe",
			name = creative_mode_defines.names.free_fluid_recipe_prefix .. fluid.name,
			category = creative_mode_defines.names.recipe_categories.free_fluids,
			ingredients = {},
			results =
			{
				{type = "fluid", name = fluid.name, amount = 5000}
			},
			main_product = fluid.name,
			subgroup = creative_mode_defines.names.item_subgroups.free_fluids,
			enabled = false
		}
	})
end

-- Some mods (e.g. Bob's Modules) add limitations to all productivity modules. But we want our super productivity module to be limitless.
data.raw["module"][creative_mode_defines.names.recipes.super_productivity_module].limitation = nil

--------------------------------------------------------------

-- All possible data raw names for items that have place results of entities.
local possible_data_raw_types_for_items_of_entities =
{
	"item",
	"item-with-entity-data",	-- 0.13
	"item-with-label",			-- 0.13
}
-- Make a dictionary for recording all items' place results, so we can find whether the items for the enemy worms and spawners already exist.
local item_place_results = {}
for _, raw_name in pairs(possible_data_raw_types_for_items_of_entities) do
	local data_raw = data.raw[raw_name]
	if data_raw then
		for _, item in pairs(data_raw) do
			if item.place_result then
				item_place_results[item.place_result] = true
			end
		end
	end
end

-- Returns whether there is already an item for the given entity.
local function has_item_for_enemy_entity(entity)
	return item_place_results[entity.name] == true
end

-- Clones the enemy entities in the given data raw and creates item and recipe for them.
-- It will not touch those entities that already have an item to build it.
local function clone_enemy_entities_in_data_raw_and_create_recipe(raw_name)
	if not data.raw[raw_name] then
		return
	end
	
	-- We will extend the data after the following for-loop, to avoid problem caused by altering the iterating list.
	local new_data = {}
	for _, entity in pairs(data.raw[raw_name]) do
		-- Add item and recipe only if there is no item for the entity.
		-- If there is already an item, there is high chance that its recipe is also there.
		if not has_item_for_enemy_entity(entity) then
			local entity_name = creative_mode_defines.names.enemy_entity_prefix .. entity.name
			local entity_localised_name
			if settings.startup[creative_mode_defines.names.settings.enemy_structures_add_name_suffix].value then
				-- Add suffix to these new spawners and worms.
				entity_localised_name = {"entity-name.creative-mode_enemy-object", "__ENTITY__" .. entity.name .. "__"}
			else
				-- Use their original name.
				entity_localised_name = {"entity-name." .. entity.name}
			end
			local item_name = creative_mode_defines.names.enemy_item_prefix .. entity.name
			local recipe_name = creative_mode_defines.names.enemy_recipe_prefix .. entity.name
			
			
			-- Clone the entity.
			-- We don't use the original entity because that will make their corpse collide with our structures.
			local new_entity = util.table.deepcopy(entity)
			new_entity.name = entity_name
			new_entity.flags = {"player-creation"} -- Reset the flags so it is blueprintable and repairable.
			new_entity.localised_name = entity_localised_name
			-- Make the entity minable.
			new_entity.minable = {hardness = 0.2, mining_time = 0.5, result = item_name}
			-- No autoplace.
			new_entity.autoplace = nil
			table.insert(new_data, new_entity)
			-- Flags.
			local flags = {"goes-to-quickbar"}
			if not settings.startup[creative_mode_defines.names.settings.unhide_items].value then
				table.insert(flags, "hidden")
			end
			-- Create item for the entity.
			table.insert(new_data,
			{
				type = "item",
				name = item_name,
				localised_name = entity_localised_name, -- Item does not know the entity's custom localised name, so we have to also use custom localised name for it.
				icon = entity.icon,
				flags = flags,
				subgroup = creative_mode_defines.names.item_subgroups.enemies,
				--order = entity.order,
				place_result = entity_name,
				stack_size = 50
			})
			-- And recipe.
			table.insert(new_data,
			{
				type = "recipe",
				name = recipe_name,
				category = creative_mode_defines.names.recipe_categories.enemies,
				ingredients = {},
				result = item_name,
				enabled = false
			})
		end
	end
	
	-- Extend data.
	data:extend(new_data)
end

-- Worms.
clone_enemy_entities_in_data_raw_and_create_recipe("turret")
-- Spawners.
clone_enemy_entities_in_data_raw_and_create_recipe("unit-spawner")

--------------------------------------------------------------

-- Absolute resistance.
local resistances = {}
for _, damage_type in pairs(data.raw["damage-type"]) do
	local damage_type_name = damage_type.name
	table.insert(resistances,
	{
		type = damage_type_name,
		percent = 100
	})
end

-- Make the super robots to have absolute resistance.
data.raw["logistic-robot"][creative_mode_defines.names.entities.super_logistic_robot].resistances = resistances
data.raw["construction-robot"][creative_mode_defines.names.entities.super_construction_robot].resistances = resistances

--------------------------------------------------------------

-- Allow the creative lab to accept all research materials.
local tools = {}
for _, tool in pairs(data.raw["tool"]) do
	table.insert(tools, tool.name)
end
data.raw["lab"][creative_mode_defines.names.entities.creative_lab].inputs = tools